/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name:  $
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.java5.authentication.jmx;

import java.security.Principal;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.remote.JMXAuthenticator;
import javax.security.auth.Subject;
import javax.security.auth.login.Configuration;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.configuration.LocalLoginContext;
import net.sf.jguard.core.principals.JMXPrincipal;
import net.sf.jguard.core.principals.UserPrincipal;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authentication.callbacks.JMXCallbackHandler;

/**
 * JGuardJMXAuthenticator is a custom JMX authenticator.
 * It logs the user connecting from JMX.
 * In jee (and jee only !), it adds a special Principal to the Subject created during the login.
 * This is a <code>JMXPrincipal</code> which keeps a reference to the classloader
 * to identify the webapp the user is login in and thus get the correct permission provider
 * from the MultipleAppPolicy.
 * @author <a href="mailto:vberetti@users.sourceforge.net">Vincent Beretti</a>
 * @since 1.0
 */
public class JGuardJMXAuthenticator implements JMXAuthenticator {

	private static final Logger logger = Logger.getLogger(JGuardJMXAuthenticator.class.getName());

	private String applicationName;
	private ClassLoader classLoader; // only used in jee
	private Configuration configuration = null;

	/**
	 * Creates a JGuardJMXAuthentication <strong>for standalone applications</strong>
	 * Retrieves the application name from system properties :
	 * <ul>
	 * <li>net.sf.jguard.application.name</li>
	 * <li>or com.sun.management.jmxremote.login.config</li>
	 * </ul>
	 */
	public JGuardJMXAuthenticator(){

		logger.info("JGuardJMXAuthentication for j2se environnement");
		String appNameProp = System.getProperty(SecurityConstants.JGUARD_APPLICATION_NAME);

		if (appNameProp != null){
			// use system property net.sf.jguard.application.name
			applicationName = appNameProp;
		}else{
			String appNameJMXProp = System.getProperty(SecurityConstants.COM_SUN_APPLICATION_NAME);
			if (appNameJMXProp != null){
				applicationName = appNameJMXProp;
			}else{
				//use default applicationName
				applicationName = CoreConstants.DEFAULT_APPLICATION_NAME;
			}
		}
		this.classLoader = null; // classLoader won't be used
	}

	/**
	 * Creates a JGuardJMXAuthenticator <strong>for jee applications</strong>
	 * @param applicationName - the webapp name
	 * @param classLoader - the classloader identifying the permissionProvider in MultipleAppPolicy
	 */
	public JGuardJMXAuthenticator(String applicationName,ClassLoader classLoader){
		logger.info("JGuardJMXAuthentication for jee environnement");
		this.applicationName = applicationName;
		this.classLoader = classLoader;
	}
	
	

	public JGuardJMXAuthenticator(String appName, ClassLoader contextClassLoader, Configuration conf) {
		logger.info("JGuardJMXAuthentication for jee environnement");
		logger.info("authentication scope is local");
		this.applicationName = appName;
		this.classLoader = contextClassLoader;
		configuration = conf;
	}

	public Subject authenticate(Object credentials) {

		Subject subject = null;
		
		if(configuration==null){
			try {
				logger.info("logging in application : " + applicationName);
				LoginContext lc = new LoginContext(applicationName, new JMXCallbackHandler(credentials));
				lc.login();
				subject = lc.getSubject();
			} catch (LoginException e) {
				logger.severe("loginException : "+e.getMessage());
				throw new SecurityException(e.getMessage());
			}
		}else{
			//'local' mode
			try {
				LocalLoginContext loginContext = new LocalLoginContext(applicationName,new JMXCallbackHandler(credentials),configuration);
				loginContext.login();
				subject = loginContext.getSubject();
			} catch (LoginException e) {
				logger.severe("loginException : "+e.getMessage());
				throw new SecurityException(e.getMessage());
			}
		}
		
		
		if (this.classLoader != null){
			// used in jee with MultipleAppPolicy
			JMXPrincipal classLoaderPrincipal = new JMXPrincipal(applicationName, this.classLoader);
			subject.getPrincipals().add(classLoaderPrincipal);
		}

		// used in ABAC permissions
		subject.getPrincipals().add(new UserPrincipal(subject));

		if (logger.isLoggable(Level.INFO)){
			logger.info("Principals set during login :");
			Set ppals = subject.getPrincipals();
			Iterator itPpals = ppals.iterator();

			while (itPpals.hasNext()){
				Principal ppal = (Principal)itPpals.next();
				logger.log(Level.INFO, ppal.toString());
			}
		}

		return subject;
	}
}
